/*
 * milka.c - Kedrigern (Milena TTS server for Raspberry Pi)
 * Copyright (C) Bohdan R. Rau 2019 <ethanak@polip.com>
 *
 * Kedrigern is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * Kedrigern is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Kedrigern.  If not, write to:
 * 	The Free Software Foundation, Inc.,
 * 	51 Franklin Street, Fifth Floor
 * 	Boston, MA  02110-1301, USA.
 */

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <milena.h>
#include <milena_mbrola.h>

#include "kedrigern_internal.h"

struct milena *milena;
struct milena_mbrola_cfg *mbrola;
static char obuf[8192];
static char ibuf[8192];
static char ixbuf[8192];
void send_outbuf(struct phone_buffer *phb,char *str)
{
    int l=strlen(str);
    if (!phb->buf_len) {
        phb->str=malloc(1024);
        phb->str_len=0;
        phb->buf_len=1024;
    }
    if (!l) return;
    while (phb->str_len+l+1 > phb->buf_len) {
        phb->buf_len*=2;
        phb->str=realloc(phb->str,phb->buf_len);
    }
    strcpy(phb->str+phb->str_len,str);
    phb->str_len+=l;
}

void translate_simple_par(char *par, struct phone_buffer *phb)
{
    int ptype;
    for (;;) {
        int n;
        n=milena_GetPhrase(milena,&par,ibuf,8192,&ptype);
        if (n) break;
        if ((ptype & 7)==7) ptype=(ptype & 0xf8) | 4;
        milena_Prestresser(milena,ibuf,obuf,8192);
        milena_TranslatePhrase(milena,(unsigned char *)obuf,ibuf,8192,0);
        milena_Poststresser(ibuf,obuf,8192);
        milena_ModMbrolaGenPhraseP(mbrola,obuf,phb,ptype);
    }
}

void initMilena(const char *udic)
{
    milena=milena_Init(
            milena_FilePath("pl_pho.dat",obuf),
            milena_FilePath("pl_dict.dat",ibuf),
            milena_FilePath("pl_stress.dat",ixbuf));
    if (!milena) {
        
        exit(1);
    }
    if (!milena_ReadPhraser(milena,
            milena_FilePath("pl_phraser.dat",ibuf))) exit(1);
    mbrola=milena_InitModMbrola(
                milena_FilePath("pl_mbrola.dat",ibuf));
    if (!mbrola) exit(1);
    if (!milena_ReadUserDic(milena,
            milena_FilePath("pl_udict.dat",ibuf))) exit(1);
    if (!milena_ReadInton(mbrola,
            milena_FilePath("pl_intona.dat",ibuf))) exit(1);
    if (!milena_ReadPhraser(milena,
                milena_FilePath("pl_hours.dat",ibuf))) exit(1);
    if (!milena_ReadPhraser(milena,
                milena_FilePath("pl_emots.dat",ibuf))) exit(1);
    if (!milena_ReadUserDicWithFlags(milena,
        KEDRIGERNPATH "/kedrigern.dic",
        MILENA_UDIC_IGNORE)) exit(1);
    if (udic && !milena_ReadUserDicWithFlags(milena, (char *)udic,
        MILENA_UDIC_IGNORE)) exit(1);
    milena_ModMbrolaSetFlag(
                mbrola,-1,MILENA_MBFL_MOUTH | MILENA_MBFL_BOOKMODE);
}

char *convertMilena(char *tresc)
{
    struct phone_buffer phb={NULL,0,0};
    int n=milena_utf2iso(tresc,NULL,1,NULL);
    if (!n) {
        fprintf(stderr,"Empty input\n");
        exit(1);
    }
    char *iso_buf = malloc(n+1);
    milena_utf2iso(tresc,iso_buf,1,NULL);
    char *psf=iso_buf, *paragraph;
    int partyp=0, lastpar=-1, pau=0;
    while (psf) {
        paragraph = psf;
        char *c = strchr(paragraph,'\n');
        if (c) {
            *c++=0;
            psf = c;
        }
        else {
            psf = NULL;
        }
        partyp=milena_GetParType(milena,&paragraph,0);
        if (!partyp) {
            if (lastpar>0) lastpar=0;
            break;
        }
        if (lastpar<0) {
            pau=0;
        }
        else {
            pau=MILENA_MBRK_NORMAL;
        }
        lastpar=partyp;
        milena_ModMbrolaBreakP(mbrola,&phb,pau);
        translate_simple_par(paragraph,&phb);
    }
    free(iso_buf);
    send_outbuf(&phb,"_ 200\n");
    return phb.str;
}
